##
##  postgresql.spec -- OpenPKG RPM Package Specification
##  Copyright (c) 2000-2025 OpenPKG Project <http://openpkg.org/>
##
##  Permission to use, copy, modify, and distribute this software for
##  any purpose with or without fee is hereby granted, provided that
##  the above copyright notice and this permission notice appear in all
##  copies.
##
##  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
##  SUCH DAMAGE.
##

#   package versions
%define       V_postgresql      17.5
%define       V_postgresql_dist 17.5
%define       V_postgresql_dir  17.5

#   package information
Name:         postgresql
Summary:      PostgreSQL Database
URL:          http://www.postgresql.org/
Vendor:       PostgreSQL Group
Packager:     OpenPKG Project
Distribution: OpenPKG Community
Class:        BASE
Group:        Database
License:      BSD
Version:      %{V_postgresql}
Release:      20250508

#   package options
%option       with_server       yes
%option       with_readline     yes
%option       with_icu          no
%option       with_threads      no
%option       with_kerberos     no
%option       with_ldap         no
%option       with_perl         no
%option       with_python       no

#   list of sources
Source0:      https://ftp.postgresql.org/pub/source/v%{V_postgresql_dir}/postgresql-%{V_postgresql_dist}.tar.bz2
Source1:      rc.postgresql
Source2:      pg_migrate
Source3:      pg_passwd
Patch0:       postgresql.patch

#   build information
BuildPreReq:  OpenPKG, openpkg >= 20160101, make, gcc, flex, bison, gzip, pkgconfig
PreReq:       OpenPKG, openpkg >= 20160101
BuildPreReq:  zlib, openssl
PreReq:       zlib, openssl
%if "%{with_readline}" == "yes"
BuildPreReq:  readline
PreReq:       readline
%endif
%if "%{with_icu}" == "yes"
BuildPreReq:  icu
PreReq:       icu
%endif
%if "%{with_kerberos}" == "yes"
BuildPreReq:  KERBEROS
PreReq:       KERBEROS
%endif
%if "%{with_ldap}" == "yes"
BuildPreReq:  openldap
PreReq:       openldap
%endif
%if "%{with_perl}" == "yes"
BuildPreReq:  perl
PreReq:       perl
%endif
%if "%{with_python}" == "yes"
BuildPreReq:  python
PreReq:       python
%endif

%description
    PostgreSQL is a sophisticated Object-Relational Database Management
    System (ORDBMS). It is fully ACID compliant and has full support
    for foreign keys, joins, views, triggers, and stored procedures
    (in multiple languages). It includes most SQL92 and SQL99 data
    types and also supports storage of binary large objects. It is
    the most advanced Open-Source RDBMS available anywhere. As an
    enterprise class RDBMS, PostgreSQL boasts sophisticated features
    such as Multi-Version Concurrency Control (MVCC), Point In Time
    Recovery (PITR), tablespaces, asynchronous replication, nested
    transactions (savepoints), online/hot backups, a sophisticated
    query planner/optimizer, and Write Ahead Logging (WAL) for fault
    tolerance. It supports international character sets, multibyte
    character encodings, Unicode, and it is locale-aware for sorting,
    case-sensitivity, and formatting. It is highly scalable both in the
    sheer quantity of data it can manage and in the number of concurrent
    users it can accommodate.

%track
    prog postgresql = {
        version   = %{V_postgresql_dist}
        url       = http://www.postgresql.org/ftp/source/
        regex     = v(\d+\.\d+(?:\.\d+)?)
    }

%prep
    %setup -q
    %patch -p0

%build
    #   adjust source tree
    %{l_shtool} subst \
        -e 's;\(#define.*DEFAULT_PGSOCKET_DIR[^"]*"\)/tmp\("\);\1%{l_prefix}/var/postgresql/run\2;' \
        src/include/pg_config_manual.h
    %{l_shtool} subst \
        -e 's;^\(sqlmansect *=\).*$;\1 7;' \
        src/makefiles/Makefile.solaris
    %{l_shtool} subst \
        -e 's;# Shared library stuff;enable_shared = yes;g' \
        src/pl/plpgsql/src/Makefile

    #   configure package
    echo "ac_cv_func_isinf=no" >config.cache
    export CC="%{l_cc}"
    export CFLAGS="%{l_cflags -O}"
    export CPPFLAGS="%{l_cppflags}"
    export LDFLAGS="%{l_ldflags}"
    export LIBS=""
%if "%{with_readline}" == "yes"
    CFLAGS="$CFLAGS `pkg-config readline --cflags-only-other`"
    CPPFLAGS="$CPPFLAGS %{l_cppflags readline .} `pkg-config readline --cflags-only-I`"
    LDFLAGS="$LDFLAGS `pkg-config readline --libs-only-l`"
    LIBS="$LIBS `pkg-config readline --libs-only-l`"
%endif
%if "%{with_kerberos}" == "yes"
    CPPFLAGS="$CPPFLAGS `krb5-config --cflags`"
    LIBS="$LIBS `krb5-config --libs`"
%endif
%if "%{with_ldap}" == "yes"
    LIBS="$LIBS -lldap -llber -lssl -lcrypto"
%endif
    export TAR="%{l_tar}"
    export YACC="bison -y"
    ./configure \
        --cache-file=./config.cache \
        --prefix=%{l_prefix} \
        --mandir=%{l_prefix}/man \
        --sysconfdir=%{l_prefix}/etc/postgresql \
        --includedir=%{l_prefix}/include/postgresql \
        --with-zlib \
        --with-openssl \
%if "%{with_readline}" == "yes"
        --with-readline \
%else
        --without-readline \
%endif
%if "%{with_icu}" == "yes"
        --with-icu \
%else
        --without-icu \
%endif
%if "%{with_threads}" == "yes"
        --enable-thread-safety \
%else
        --disable-thread-safety \
%endif
%if "%{with_kerberos}" == "yes"
        --with-krb5 \
        --with-krb-srvnam=postgresql \
%endif
%if "%{with_ldap}" == "yes"
        --with-ldap \
%endif
%if "%{with_perl}" == "yes"
        --with-perl \
%endif
%if "%{with_python}" == "yes"
        --with-python \
%endif
        --disable-shared

    #   build package
    %{l_make} %{l_mflags}

    #   build manual pages
    (   cd doc/src/sgml
        %{l_make} %{l_mflags} man
    ) || exit $?

    #   rebuild pg_config with hard-coded path to avoid that it provides
    #   dynamically resolved paths which circumvent symlinks, etc.
    ( cd src/bin/pg_config
      %{l_shtool} subst \
          -e 's:find_my_exec(argv.0., mypath):0; strcpy(mypath, "%{l_prefix}/bin/pg_config"):' \
          pg_config.c
      %{l_make} %{l_mflags}
    ) || exit $?

%install
    #   perform standard installation procedure
    cp /dev/null register.txt
    %{l_make} %{l_mflags} install DESTDIR=$RPM_BUILD_ROOT
    (   cd doc/src/sgml
        touch man-stamp
        %{l_make} %{l_mflags} install-man DESTDIR=$RPM_BUILD_ROOT
    ) || exit $?

    #   strip down installation
    rm -rf $RPM_BUILD_ROOT%{l_prefix}/share/doc
    strip $RPM_BUILD_ROOT%{l_prefix}/bin/* >/dev/null 2>&1 || true
    rm -f $RPM_BUILD_ROOT%{l_prefix}/man/man1/pgaccess.1
    rm -f $RPM_BUILD_ROOT%{l_prefix}/man/man1/pgtclsh.1
    rm -f $RPM_BUILD_ROOT%{l_prefix}/man/man1/pgtksh.1

    #   namespace adjustments to installation
    for prog in \
        createdb createuser dropdb clusterdb \
        dropuser initdb vacuumdb reindexdb; do
        mv  $RPM_BUILD_ROOT%{l_prefix}/bin/$prog \
            $RPM_BUILD_ROOT%{l_prefix}/bin/pg_$prog
        mv  $RPM_BUILD_ROOT%{l_prefix}/man/man1/$prog.1 \
            $RPM_BUILD_ROOT%{l_prefix}/man/man1/pg_$prog.1
    done
    ( cd $RPM_BUILD_ROOT%{l_prefix}/man/man7
      for man in *.7; do
          mv $man pg_$man
      done
    ) || exit $?

    #   create additional directories
    %{l_shtool} mkdir -f -p -m 755 \
        $RPM_BUILD_ROOT%{l_prefix}/var/postgresql/db \
        $RPM_BUILD_ROOT%{l_prefix}/var/postgresql/run \
        $RPM_BUILD_ROOT%{l_prefix}/var/postgresql/log

    #   install addon utilities
    %{l_shtool} install -c -m 755 %{l_value -s -a} \
        %{SOURCE pg_migrate} $RPM_BUILD_ROOT%{l_prefix}/bin/
    %{l_shtool} install -c -m 755 %{l_value -s -a} \
        -e "s;@l_bash@;%{l_bash};g" \
        %{SOURCE pg_passwd} $RPM_BUILD_ROOT%{l_prefix}/bin/

    #   adjust default configuration for hourly auto-vacuum operation
    %{l_shtool} subst \
        -e 's;^# *\(track_counts *=\) *[^#]*\(#.*\);\1 on \2;' \
        -e 's;^# *\(autovacuum *=\) *[^#]*\(#.*\);\1 on \2;' \
        $RPM_BUILD_ROOT%{l_prefix}/share/postgresql/postgresql.conf.sample

    #   install run-command script
    %{l_shtool} mkdir -f -p -m 755 \
        $RPM_BUILD_ROOT%{l_prefix}/etc/rc.d
    %{l_shtool} install -c -m 755 %{l_value -s -a} \
        %{SOURCE rc.postgresql} \
        $RPM_BUILD_ROOT%{l_prefix}/etc/rc.d/

    #   optionally strip down to client-only installation
%if "%{with_server}" != "yes"
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/bin/pg_[a-bd-z]*
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/bin/pg_c[a-np-z]*
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/bin/pg_controldata
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/bin/post*
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/man/man1/pg_[a-bd-z]*
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/man/man1/pg_c[a-np-z]*
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/man/man1/pg_controldata.1
    rm -f  $RPM_BUILD_ROOT%{l_prefix}/man/man1/post*
    rm -rf $RPM_BUILD_ROOT%{l_prefix}/etc/rc.d
    rm -rf $RPM_BUILD_ROOT%{l_prefix}/include/postgresql/server
    rm -rf $RPM_BUILD_ROOT%{l_prefix}/lib/postgresql
    rm -rf $RPM_BUILD_ROOT%{l_prefix}/share/postgresql
    rm -rf $RPM_BUILD_ROOT%{l_prefix}/var/postgresql
%endif

    #   determine installation files
    %{l_rpmtool} files -v -ofiles -r$RPM_BUILD_ROOT \
%if "%{with_server}" == "yes"
        %{l_files_std} \
        '%attr(700,%{l_rusr},%{l_rgrp}) %dir %{l_prefix}/var/postgresql/db' \
        '%attr(755,%{l_rusr},%{l_rgrp}) %dir %{l_prefix}/var/postgresql/run' \
        '%attr(755,%{l_rusr},%{l_rgrp}) %dir %{l_prefix}/var/postgresql/log'
%else
        %{l_files_std}
%endif

%files -f files

%clean

%pre
%if "%{with_server}" == "yes"
    #   before upgrade, check migration dump, save status and stop service
    [ $1 -eq 2 ] || exit 0
    if [ -f $RPM_INSTALL_PREFIX/var/postgresql/db/PG_VERSION -a \
         -f $RPM_INSTALL_PREFIX/bin/pg_migrate ]; then
        #   database migration dumping hint
        v_old_all=`cat $RPM_INSTALL_PREFIX/var/postgresql/db/PG_VERSION`
        v_old_maj=`echo "$v_old_all" | sed -e 's/^\([0-9]*\)\..*/\1/'`
        v_new_all="%{V_postgresql}"
        v_new_maj=`echo "$v_new_all" | sed -e 's/^\([0-9]*\)\..*/\1/'`
        if [ ".$v_old_maj" != ".$v_new_maj" ]; then
            ( echo "You are upgrading from PostgreSQL $v_old_maj to PostgresSQL $v_new_maj,"
              echo "which is a major version change. We expect a database incompatibility,"
              echo "so a full database backup and restore is required!"
            ) | %{l_rpmtool} msg -b -t notice
            if [ ".$RPM_POSTGRESQL_MIGRATE" != .ignore ]; then
                if [ ! -f $RPM_INSTALL_PREFIX/var/postgresql/db.dump.sql.bz2 ]; then
                    ( echo "We are performing a full backup of your existing database"
                      echo "($RPM_INSTALL_PREFIX/var/postgresql/db/) for you by running:"
                      echo "    \$ $RPM_INSTALL_PREFIX/bin/pg_migrate dump"
                      echo "If this fails for some reasons, try to dump your data manually:"
                      echo "    \$ $RPM_INSTALL_PREFIX/bin/pg_dumpall -U postgresql -o | \\ "
                      echo "      $RPM_INSTALL_PREFIX/lib/openpkg/bzip2 -9 \\ "
                      echo "      >$RPM_INSTALL_PREFIX/var/postgresql/db.dump.sql.bz2"
                      echo "Alternatively, if you want to force this package to be installed without"
                      echo "a previously created database dump, run the following command"
                      echo "before trying this package upgrade again:"
                      echo "    \$ RPM_POSTGRESQL_MIGRATE=ignore; export RPM_POSTGRESQL_MIGRATE"
                    ) | %{l_rpmtool} msg -b -t notice
                    $RPM_INSTALL_PREFIX/bin/pg_migrate dump
                    if [ $? -ne 0 ] || [ ! -f $RPM_INSTALL_PREFIX/var/postgresql/db.dump.sql.bz2 ]; then
                        ( echo "Automatic database dump creation failed!"
                          echo "PLEASE INVESTIGATE MANUALLY YOURSELF!"
                        ) | %{l_rpmtool} msg -b -t error
                        exit 1
                    fi
                fi
            fi
        fi
    fi
    eval `%{l_rc} postgresql status 2>/dev/null | tee %{l_tmpfile}`
    %{l_rc} postgresql stop 2>/dev/null
    exit 0
%endif

%post
%if "%{with_server}" == "yes"
    l_pguser="postgresql"
    l_pgpass="postgresql"
    if [ $1 -eq 1 ]; then
        #   create initial database
        su - %{l_rusr} -c \
            "LC_CTYPE=C; export LC_CTYPE; umask 077; \
            rm -rf $RPM_INSTALL_PREFIX/var/postgresql/db/*; \
            echo $l_pgpass >$RPM_INSTALL_PREFIX/var/postgresql/run/pg_initdb.pw; \
            $RPM_INSTALL_PREFIX/bin/pg_initdb \
                --encoding=SQL_ASCII --locale=C --auth=md5 --username=$l_pguser \
                --pwfile=$RPM_INSTALL_PREFIX/var/postgresql/run/pg_initdb.pw \
                --pgdata=$RPM_INSTALL_PREFIX/var/postgresql/db; \
            rm -f $RPM_INSTALL_PREFIX/var/postgresql/run/pg_initdb.pw" 2>&1 | \
        $RPM_INSTALL_PREFIX/lib/openpkg/shtool prop \
            -p "Creating initial PostgreSQL DB in $RPM_INSTALL_PREFIX/var/postgresql/db"
        if [ ! -f $RPM_INSTALL_PREFIX/var/postgresql/db/PG_VERSION ]; then
            echo "ERROR: failed to create initial PostgreSQL database" 1>&2
            exit 1
        fi
        ( umask 077
          ( echo "##"
            echo "##  pg_superuser.conf -- PostgreSQL database superuser configuration"
            echo "##"
            echo ""
            echo "superuser_database=\"template1\""
            echo "superuser_username=\"$l_pguser\""
            echo "superuser_password=\"$l_pgpass\""
            echo ""
          ) >$RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf
          chown %{l_rusr}:%{l_rgrp} $RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf || exit $?
          chmod 600 $RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf || exit $?
        ) || exit $?

        #   display information about next steps
        ( echo "An initial PostgreSQL DB was created with the two standard"
          echo "databases 'template0' and 'template1'. The owner of both"
          echo "is the DB user '$l_pguser'. Its initial password is '$l_pgpass'."
          echo ""
          echo "After starting PostgreSQL with"
          echo "    \$ $RPM_INSTALL_PREFIX/bin/openpkg rc postgresql start"
          echo "you should immediately change this with the following command:"
          echo "    \$ $RPM_INSTALL_PREFIX/bin/pg_passwd postgresql template1"
          echo ""
          echo "Then you usually create a database <database> for a user <user>"
          echo "and with password <password>:"
          echo "    \$ $RPM_INSTALL_PREFIX/bin/psql -U $l_pguser -d template1"
          echo "    template1=> CREATE ROLE <user>"
          echo "                LOGIN ENCRYPTED PASSWORD '<password>'"
          echo "                NOCREATEDB NOCREATEROLE;"
          echo "    template1=> CREATE DATABASE <database> OWNER <user>;"
          echo "    template1=> \\q"
          echo ""
          echo "Or you can create a database for a user <user> (assuming that"
          echo "his home directory is /u/<user>) with password <password> under"
          echo "path /u/<user>/rdbms with the commands:"
          echo "    \$ mkdir /u/<user>/rdbms"
          echo "    \$ chmod 700 /u/<user>/rdbms"
          echo "    \$ chown %{l_rusr}:%{l_rgrp} /u/<user>/rdbms"
          echo "    \$ $RPM_INSTALL_PREFIX/bin/psql -U $l_pguser -d template1"
          echo "    template1=> CREATE ROLE <user>"
          echo "                LOGIN ENCRYPTED PASSWORD '<password>'"
          echo "                NOCREATEDB NOCREATEROLE;"
          echo "    template1=> CREATE TABLESPACE <user> OWNER <user>"
          echo "                LOCATION '/u/<user>/rdbms';"
          echo "    template1=> CREATE DATABASE <user> OWNER <user>"
          echo "                TABLESPACE <user>;"
          echo "    template1=> \\q"
          echo ""
          echo "The user <user> will be able to connect to his database with:"
          echo "    \$ echo 'localhost:*:<database>:<user>:<password>' >>~/.pgpass"
          echo "    \$ chmod 600 ~/.pgpass"
          echo "    \$ $RPM_INSTALL_PREFIX/bin/psql"
        ) | %{l_rpmtool} msg -b -t notice
    fi

    if [ $1 -eq 2 ]; then
        #   after upgrade, restore status
        { eval `cat %{l_tmpfile}`; rm -f %{l_tmpfile}; true; } >/dev/null 2>&1
        [ ".$postgresql_active" = .yes ] && %{l_rc} postgresql start
        if [   -f $RPM_INSTALL_PREFIX/var/postgresql/db/PG_VERSION -a \
             ! -f $RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf ]; then
            ( umask 077
              ( echo "##"
                echo "##  pg_superuser.conf -- PostgreSQL database superuser configuration"
                echo "##"
                echo ""
                echo "superuser_database=\"template1\""
                echo "superuser_username=\"$l_pguser\""
                echo "superuser_password=\"\""
                echo ""
              ) >$RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf
              chown %{l_rusr}:%{l_rgrp} $RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf || exit $?
              chmod 600 $RPM_INSTALL_PREFIX/var/postgresql/db/pg_superuser.conf || exit $?
            ) || exit $?
            ( echo "Created still missing \"pg_superuser.conf\" configuration file."
              echo "You should update its content by resetting the PostgreSQL"
              echo "superuser account password with the following command:"
              echo "    \$ $RPM_INSTALL_PREFIX/bin/pg_passwd postgresql template1"
            ) | %{l_rpmtool} msg -b -t warn
        fi
        if [ -f $RPM_INSTALL_PREFIX/var/postgresql/db/PG_VERSION -a ".$PG_MIGRATE" != .ignore ]; then
            #   database migration restoring hint
            v_old_all=`cat $RPM_INSTALL_PREFIX/var/postgresql/db/PG_VERSION`
            v_old_maj=`echo "$v_old_all" | sed -e 's/^\([0-9]*\.[0-9]*\).*/\1/'`
            v_new_all="%{V_postgresql}"
            v_new_maj=`echo "$v_new_all" | sed -e 's/^\([0-9]*\.[0-9]*\).*/\1/'`
            if [ ".$v_old_maj" != ".$v_new_maj" ]; then
                ( echo "You upgraded from PostgreSQL $v_old_all to PostgresSQL $v_new_all,"
                  echo "which is a major version upgrade. We expect a database incompatibility,"
                  echo "so we strongly recommend you to recreate the existing database under"
                  echo "$RPM_INSTALL_PREFIX/var/postgresql/db/ by running the following command:"
                  echo "    \$ $RPM_INSTALL_PREFIX/bin/pg_migrate restore"
                  echo "If this fails for some reasons, try to restore your data manually:"
                  echo "    \$ $RPM_INSTALL_PREFIX/lib/openpkg/bzip2 -d -c \\ "
                  echo "      $RPM_INSTALL_PREFIX/var/postgresql/db.dump.sql.bz2 | \\ "
                  echo "      $RPM_INSTALL_PREFIX/bin/psql -U postgresql -d template1"
                ) | %{l_rpmtool} msg -b -t warn
            fi
        fi
    fi
    exit 0
%endif

%preun
%if "%{with_server}" == "yes"
    #   before erase, stop service and remove log files
    [ $1 -eq 0 ] || exit 0
    %{l_rc} postgresql stop 2>/dev/null
    rm -f $RPM_INSTALL_PREFIX/var/postgresql/run/* >/dev/null 2>&1 || true
    rm -f $RPM_INSTALL_PREFIX/var/postgresql/log/* >/dev/null 2>&1 || true
    exit 0
%endif

